'use strict';
Object.defineProperty(exports, '__esModule', {value: true});
exports.Deferred = void 0;
/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @providesModule Deferred
 * @typechecks
 *
 */

/**
 * Deferred provides a Promise-like API that exposes methods to resolve and
 * reject the Promise. It is most useful when converting non-Promise code to use
 * Promises.
 *
 * If you want to export the Promise without exposing access to the resolve and
 * reject methods, you should export `getPromise` which returns a Promise with
 * the same semantics excluding those methods.
 */
class Deferred {
  constructor() {
    this._settled = false;
    this._promise = new Promise((resolve, reject) => {
      this._resolve = resolve;
      this._reject = reject;
    });
  }

  getPromise() {
    return this._promise;
  }

  resolve(value) {
    this._settled = true;

    this._resolve(value);
  }

  reject(reason) {
    this._settled = true;

    this._reject(reason);
  }

  catch(onReject) {
    return Promise.prototype.catch.apply(this._promise, arguments);
  }

  then(onFulfill, onReject) {
    return Promise.prototype.then.apply(this._promise, arguments);
  }

  done(onFulfill, onReject) {
    // Embed the polyfill for the non-standard Promise.prototype.done so that
    // users of the open source fbjs don't need a custom lib for Promise
    const promise = arguments.length
      ? this._promise.then.apply(this._promise, arguments)
      : this._promise;
    promise.then(undefined, (err) => {
      setTimeout(() => {
        throw err;
      }, 0);
    });
  }

  isSettled() {
    return this._settled;
  }
}
exports.Deferred = Deferred;
