'use strict';
Object.defineProperty(exports, '__esModule', {value: true});
exports.default = main;

var _rsocketCore = require('rsocket-core');
var _rsocketTcpClient = _interopRequireDefault(require('rsocket-tcp-client'));
var _RSocketTckRequestResponseSubscriber = _interopRequireDefault(
  require('./RSocketTckRequestResponseSubscriber')
);
var _RSocketTckRequestStreamSubscriber = _interopRequireDefault(
  require('./RSocketTckRequestStreamSubscriber')
);

var _chalk = _interopRequireDefault(require('chalk'));
var _fs = _interopRequireDefault(require('fs'));
var _path = _interopRequireDefault(require('path'));
var _util = _interopRequireDefault(require('util'));
var _yargs = _interopRequireDefault(require('yargs'));
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }
  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}
function _asyncToGenerator(fn) {
  return function () {
    var self = this,
      args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);
      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, 'next', value);
      }
      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, 'throw', err);
      }
      _next(undefined);
    });
  };
}

function sprintf(format, ...args) {
  let index = 0;
  return format.replace(/%s/g, () => '' + args[index++]);
}

function nullthrows(x) {
  if (x != null) {
    return x;
  }
  throw new Error('got null or undefined');
}

function areEqual(a, b) {
  return JSON.stringify(a) === JSON.stringify(b);
}

function main() {
  const argv = _yargs.default
    .usage('$0 --testfile <path> --host <host> --port <port> [--verbose]')
    .options({
      host: {
        default: '127.0.0.1',
        describe: 'TCK server hostname.',
        type: 'string',
      },

      port: {
        default: 9898,
        describe: 'TCK server port.',
        type: 'string',
      },

      testfile: {
        default: _path.default.join(__dirname, './clienttest.txt'),
        describe: 'Path to client test file.',
        type: 'string',
      },

      verbose: {
        default: false,
        describe:
          'Log each action as it is performed instead of just logging success/failure.',
        type: 'boolean',
      },
    })
    .help().argv;

  Promise.resolve(run(argv)).then(
    () => {
      process.stdout.write(_chalk.default.green('All tests pass.') + '\n');
      process.exit(0);
    },
    (error) => {
      process.stderr.write(
        _chalk.default.red('Test failed: ' + error.message) + '\n'
      );
      process.exit(1);
    }
  );
}
function run(_x) {
  return _run.apply(this, arguments);
}
function _run() {
  _run = _asyncToGenerator(function* (options) {
    const testfilePath = _path.default.resolve(process.cwd(), options.testfile);
    if (!_fs.default.existsSync(testfilePath)) {
      throw new Error(
        `--testfile ${options.testfile}: file does does not exist (${testfilePath}).`
      );
    }
    process.stdout.write(_chalk.default.inverse('Running TCK tests') + '\n');
    process.stdout.write(`Using testfile${testfilePath}\n`);
    process.stdout.write(
      `Connecting to server at ${options.host}:${options.port}\n`
    );

    const testSource = _fs.default.readFileSync(testfilePath, 'utf8');
    const testCases = testSource
      .split('!')
      .filter((item) => item.trim().length > 0);

    const socket = yield connect(options);

    function assert(cond, msg, ...rest) {
      if (!cond) {
        throw new Error(sprintf(msg, ...rest));
      }
    }
    function log(msg, ...rest) {
      if (options.verbose) {
        process.stdout.write(_chalk.default.dim(sprintf(msg, ...rest)) + '\n');
      }
    }

    try {
      while (testCases.length) {
        const testCase = testCases.shift();
        if (!testCase.length) {
          continue;
        }
        let subscriber;

        const lines = testCase.split('\n');
        while (lines.length) {
          const line = lines.shift().trim();
          if (!line.length) {
            continue;
          }
          const [command, ...args] = line.split('%%');
          log(command + ' ' + JSON.stringify(args));
          switch (command) {
            case 'name': {
              break;
            }
            case 'subscribe': {
              const [type, _, data, metadata] = args; // eslint-disable-line no-unused-vars
              if (type === 'rr') {
                subscriber = new _RSocketTckRequestResponseSubscriber.default(
                  log
                );
                socket.requestResponse({data, metadata}).subscribe(subscriber);
              } else if (type === 'rs') {
                subscriber = new _RSocketTckRequestStreamSubscriber.default(
                  log
                );
                socket.requestStream({data, metadata}).subscribe(subscriber);
              } else {
                assert(false, 'Invalid `subscribe` type %s.', type);
              }
              break;
            }
            case 'request': {
              const [n] = args;
              nullthrows(subscriber).request(parseInt(n, 10));
              break;
            }
            case 'cancel': {
              nullthrows(subscriber).cancel();
              break;
            }
            case 'await': {
              const [type, _, nOrTime] = args; // eslint-disable-line no-unused-vars
              if (type === 'terminal') {
                yield nullthrows(subscriber).awaitTerminal();
              } else if (type === 'atLeast') {
                yield nullthrows(subscriber).awaitN(parseInt(nOrTime, 10));
              } else if (type === 'no_events') {
                yield delay(parseInt(nOrTime, 10));
              } else {
                assert(false, 'Invalid `await` type %s.', type);
              }
              break;
            }
            case 'take': {
              const [n] = args;
              yield nullthrows(subscriber).awaitN(parseInt(n, 10));
              nullthrows(subscriber).cancel();
              break;
            }
            case 'assert': {
              const [type, _, other] = args; // eslint-disable-line no-unused-vars
              if (type === 'no_error') {
                assert(
                  !nullthrows(subscriber).hasError(),
                  'Expected onError not to be called.'
                );
              } else if (type === 'error') {
                assert(
                  nullthrows(subscriber).hasError(),
                  'Expected onError to be called.'
                );
              } else if (type === 'received') {
                const expected = parsePayloads(other);
                const actual = nullthrows(subscriber).getPayloads();
                if (!areEqual(actual, expected)) {
                  log('expected: %s', _util.default.inspect(expected));
                  log('actual: %s', _util.default.inspect(actual));
                  assert(false, 'Actual/expected payloads differed.');
                }
              } else if (type === 'received_n') {
                const expected = parseInt(other, 10);
                const actual = nullthrows(subscriber).getPayloads().length;
                assert(
                  actual === expected,
                  'Expected exactly %s payloads, got %s.',
                  expected,
                  actual
                );
              } else if (type === 'received_at_least') {
                const expected = parseInt(other, 10);
                const actual = nullthrows(subscriber).getPayloads().length;
                assert(
                  actual >= expected,
                  'Expected at least %s payloads, got %s.',
                  expected,
                  actual
                );
              } else if (type === 'completed') {
                assert(
                  nullthrows(subscriber).isCompleted(),
                  'Expected onComplete to be called.'
                );
              } else if (type === 'no_completed') {
                assert(
                  !nullthrows(subscriber).isCompleted(),
                  'Expected onComplete not to be called.'
                );
              } else if (type === 'canceled') {
                assert(
                  nullthrows(subscriber).isCanceled(),
                  'Expected request to be canceled.'
                );
              }
              break;
            }
            case 'EOF':
              return;
            default:
              assert(false, 'Unsupported command %s', command);
          }
        }
      }
    } catch (error) {
      log(error.stack || error.message);
      throw error;
    }
  });
  return _run.apply(this, arguments);
}
function connect(_x2) {
  return _connect.apply(this, arguments);
}
function _connect() {
  _connect = _asyncToGenerator(function* (options) {
    const client = new _rsocketCore.RSocketClient({
      setup: {
        dataMimeType: 'text/plain',
        keepAlive: 1000000, // avoid sending during test
        lifetime: 100000,
        metadataMimeType: 'text/plain',
      },

      transport: new _rsocketTcpClient.default({
        host: options.host,
        port: options.port,
      }),
    });

    return new Promise((resolve, reject) => {
      client.connect().subscribe({
        onComplete: resolve,
        onError: reject,
      });
    });
  });
  return _connect.apply(this, arguments);
}

function parsePayloads(data) {
  const payloads = [];
  data.split('&&').forEach((item) => {
    const [data, metadata] = item.split(',');
    if (data != null && metadata != null) {
      payloads.push({data, metadata});
    }
  });
  return payloads;
}
function delay(_x3) {
  return _delay.apply(this, arguments);
}
function _delay() {
  _delay = _asyncToGenerator(function* (ms) {
    return new Promise((resolve) => {
      setTimeout(resolve, ms);
    });
  });
  return _delay.apply(this, arguments);
}
